from aiohttp import web

from unv.utils.files import calc_crc32_for_file

from .deploy import SETTINGS as DEPLOY_SETTINGS


async def render_template(
        request, template_name, context=None, status=web.HTTPOk.status_code):
    template = request.app['jinja2'].get_template(template_name)
    return web.Response(
        text=await template.render(context or {}),
        status=status, charset='utf-8',
        content_type='text/html'
    )


def url_for_static(path: str, private: bool = False, with_hash: bool = False):
    url = DEPLOY_SETTINGS.static_public_url
    directory = DEPLOY_SETTINGS.static_public_dir

    if private:
        url = DEPLOY_SETTINGS.static_private_url
        directory = DEPLOY_SETTINGS.static_private_dir

    real_path = directory / path.lstrip('/')
    hash_ = ''
    if with_hash:
        hash_ = '?hash={}'.format(calc_crc32_for_file(real_path))
    path = str(path).replace(str(directory), '', 1).lstrip('/')
    return f"{url}/{path}{hash_}"


def url_with_domain(path: str):
    protocol = 'http'
    path = path.lstrip('/')
    if DEPLOY_SETTINGS.use_https:
        protocol = 'https'
    return f'{protocol}://{DEPLOY_SETTINGS.domain}/{path}'


def make_url_for_func(app):
    def url_for(route, with_domain=False, **parts):
        parts = {key: str(value) for key, value in parts.items()}
        url = app.router[route].url_for(**parts)
        if with_domain:
            url = url_with_domain(url)
        return url
    return url_for


def inline_static_from(path, private=False):
    directory = DEPLOY_SETTINGS.static_public_dir
    if private:
        directory = DEPLOY_SETTINGS.static_private_dir

    with (directory / path).open('r') as f:
        return f.read().replace("\n", "")



import jinja2

from aiohttp import web

from unv.app.settings import SETTINGS as APP_SETTINGS

from .helpers import (
    url_for_static, url_with_domain, inline_static_from, make_url_for_func
)
from .settings import SETTINGS


def setup_jinja2(app: web.Application):
    if not SETTINGS.jinja2_enabled:
        return

    app['jinja2'] = jinja2.Environment(**SETTINGS.jinja2_settings)
    app['jinja2'].globals.update({
        'url_for': make_url_for_func(app),
        'url_for_static': url_for_static,
        'url_with_domain': url_with_domain,
        'inline_static_from': inline_static_from,
        'for_developemnt': APP_SETTINGS.is_development,
        'for_production': APP_SETTINGS.is_production,
        'for_testing': APP_SETTINGS.is_testing
    })


def setup(app: web.Application):
    setup_jinja2(app)



import os
import glob
import pathlib
import asyncio

import uvloop

from aiohttp import web

from unv.app.settings import SETTINGS as APP_SETTINGS

from .deploy import SETTINGS as DEPLOY_SETTINGS


def link_component_static_dirs(component):
    component_path = pathlib.Path(
        os.path.realpath(component.__file__)).parent
    static_path = component_path / 'static'
    public_dir = DEPLOY_SETTINGS.static_public_dir
    private_dir = DEPLOY_SETTINGS.static_private_dir

    public_app_dirs = str(static_path / public_dir.name / '*')
    for directory in glob.iglob(public_app_dirs):
        os.system('mkdir -p {}'.format(public_dir))
        os.system('ln -sf {} {}'.format(directory, public_dir))

    private_app_dirs = str(static_path / private_dir.name / '*')
    for directory in glob.iglob(private_app_dirs):
        os.system('mkdir -p {}'.format(private_dir))
        os.system('ln -sf {} {}'.format(directory, private_dir))


def create_app(link_static: bool = False):
    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
    app = web.Application()

    for component in APP_SETTINGS.get_components():
        component.setup(app)
        if link_static:
            link_component_static_dirs(component)

    return app


def run_app(app):
    web.run_app(
        app, host=DEPLOY_SETTINGS.host,
        port=DEPLOY_SETTINGS.port + DEPLOY_SETTINGS.instance,
        access_log=None
    )



import functools

import ujson as json

from aiohttp import web

from .helpers import render_template


def as_json(f):
    """Return json response from passed dict result."""
    @functools.wraps(f)
    async def wrapper(request, *args, **kwargs):
        result = await f(request, *args, **kwargs)
        return web.json_response(result, dumps=json.dumps)
    return wrapper


def render(
        template_name: str, default_context: dict = None,
        status: int = web.HTTPOk.status_code):
    """Render jinja2 template by given name and custom context processors."""
    default_context = default_context or {}

    def decorator(f):
        @functools.wraps(f)
        async def wrapper(request, *args, **kwargs):
            context = default_context.copy()
            result = await f(request, *args, **kwargs)
            context.update(result)
            return await render_template(
                request, template_name, context, status)
        return wrapper

    return decorator



import copy

import jinja2

from unv.app.core import ComponentSettings
from unv.app.settings import SETTINGS as APP_SETTINGS


class WebSettings(ComponentSettings):
    KEY = 'web'
    SCHEMA = {
        'autoreload': {'type': 'boolean', 'required': True},
        'jinja2': {
            'type': 'dict',
            'required': True,
            'schema': {'enabled': {'type': 'boolean'}}
        }
    }
    DEFAULT = {
        'autoreload': False,
        'jinja2': {'enabled': True},
    }

    @property
    def jinja2_enabled(self):
        return self._data['jinja2']['enabled']

    @property
    def jinja2_settings(self):
        settings = copy.deepcopy(self._data.get('jinja2', {}))
        settings.pop('enabled')
        settings['enable_async'] = True
        settings['loader'] = jinja2.ChoiceLoader([
            jinja2.PackageLoader(component.__name__)
            for component in APP_SETTINGS.get_components()
        ])
        if 'jinja2.ext.i18n' not in settings.setdefault('extensions', []):
            settings['extensions'].append('jinja2.ext.i18n')
        return settings


SETTINGS = WebSettings()
